@@ -21,7 +21,9 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <sched.h>
+#include <spawn.h>
 #include <stdio.h>
+#include <sys/socket.h>
 #include <sys/types.h>
 #include <sys/wait.h>
 
@@ -168,7 +170,36 @@ void debug_at_flags(FILE *f, int flags) {
 }
 
 /**
- * Debug-print the 'cmd' parameter of an fcntl() call.
+ * Debug spawn‐flags attribute (set using posix_spawnattr_setflags(3)).
+ */
+void debug_psfa_attr_flags(FILE *f, int flags) {
+  DEBUG_BITMAP_START(f, flags)
+#ifdef POSIX_SPAWN_RESETIDS
+  DEBUG_BITMAP_FLAG(f, flags, POSIX_SPAWN_RESETIDS)
+#endif
+#ifdef POSIX_SPAWN_SETPGROUP
+  DEBUG_BITMAP_FLAG(f, flags, POSIX_SPAWN_SETPGROUP)
+#endif
+#ifdef POSIX_SPAWN_SETSIGDEF
+  DEBUG_BITMAP_FLAG(f, flags, POSIX_SPAWN_SETSIGDEF)
+#endif
+#ifdef POSIX_SPAWN_SETSIGMASK
+  DEBUG_BITMAP_FLAG(f, flags, POSIX_SPAWN_SETSIGMASK)
+#endif
+#ifdef POSIX_SPAWN_SETEXEC
+  DEBUG_BITMAP_FLAG(f, flags, POSIX_SPAWN_SETEXEC)
+#endif
+#ifdef POSIX_SPAWN_START_SUSPENDED
+  DEBUG_BITMAP_FLAG(f, flags, POSIX_SPAWN_START_SUSPENDED)
+#endif
+#ifdef POSIX_SPAWN_CLOEXEC_DEFAULT
+  DEBUG_BITMAP_FLAG(f, flags, POSIX_SPAWN_CLOEXEC_DEFAULT)
+#endif
+  DEBUG_BITMAP_END_HEX(f, flags)
+}
+
+/**
+ * Debug-print the 'cmdd' parameter of an fcntl() call.
  */
 void debug_fcntl_cmd(FILE *f, int cmd) {
   DEBUG_VALUE_START(f, cmd)
@@ -227,6 +258,144 @@ void debug_fcntl_cmd(FILE *f, int cmd) {
 #endif
 #ifdef F_SET_FILE_RW_HINT
   DEBUG_VALUE_VALUE(f, cmd, F_SET_FILE_RW_HINT)
+#endif
+#ifdef F_FLUSH_DATA
+  DEBUG_VALUE_VALUE(f, cmd, F_FLUSH_DATA)
+#endif
+#ifdef F_CHKCLEAN
+  DEBUG_VALUE_VALUE(f, cmd, F_CHKCLEAN)
+#endif
+#ifdef F_PREALLOCATE
+  DEBUG_VALUE_VALUE(f, cmd, F_PREALLOCATE)
+#endif
+#ifdef F_SETSIZE
+  DEBUG_VALUE_VALUE(f, cmd, F_SETSIZE)
+#endif
+#ifdef F_RDADVISE
+  DEBUG_VALUE_VALUE(f, cmd, F_RDADVISE)
+#endif
+#ifdef F_RDAHEAD
+  DEBUG_VALUE_VALUE(f, cmd, F_RDAHEAD)
+#endif
+#ifdef F_NOCACHE
+  DEBUG_VALUE_VALUE(f, cmd, F_NOCACHE)
+#endif
+#ifdef F_LOG2PHYS
+  DEBUG_VALUE_VALUE(f, cmd, F_LOG2PHYS)
+#endif
+#ifdef F_GETPATH
+  DEBUG_VALUE_VALUE(f, cmd, F_GETPATH)
+#endif
+#ifdef F_FULLFSYNC
+  DEBUG_VALUE_VALUE(f, cmd, F_FULLFSYNC)
+#endif
+#ifdef F_PATHPKG_CHECK
+  DEBUG_VALUE_VALUE(f, cmd, F_PATHPKG_CHECK)
+#endif
+#ifdef F_FREEZE_FS
+  DEBUG_VALUE_VALUE(f, cmd, F_FREEZE_FS)
+#endif
+#ifdef F_THAW_FS
+  DEBUG_VALUE_VALUE(f, cmd, F_THAW_FS)
+#endif
+#ifdef F_ADDSIGS
+  DEBUG_VALUE_VALUE(f, cmd, F_ADDSIGS)
+#endif
+#ifdef F_ADDFILESIGS
+  DEBUG_VALUE_VALUE(f, cmd, F_ADDFILESIGS)
+#endif
+#ifdef F_NODIRECT
+  DEBUG_VALUE_VALUE(f, cmd, F_NODIRECT)
+#endif
+#ifdef F_GETPROTECTIONCLASS
+  DEBUG_VALUE_VALUE(f, cmd, F_GETPROTECTIONCLASS)
+#endif
+#ifdef F_SETPROTECTIONCLASS
+  DEBUG_VALUE_VALUE(f, cmd, F_SETPROTECTIONCLASS)
+#endif
+#ifdef F_LOG2PHYS_EXT
+  DEBUG_VALUE_VALUE(f, cmd, F_LOG2PHYS_EXT)
+#endif
+#ifdef F_GETLKPID
+  DEBUG_VALUE_VALUE(f, cmd, F_GETLKPID)
+#endif
+#ifdef F_SETBACKINGSTORE
+  DEBUG_VALUE_VALUE(f, cmd, F_SETBACKINGSTORE)
+#endif
+#ifdef F_GETPATH_MTMINFO
+  DEBUG_VALUE_VALUE(f, cmd, F_GETPATH_MTMINFO)
+#endif
+#ifdef F_GETCODEDIR
+  DEBUG_VALUE_VALUE(f, cmd, F_GETCODEDIR)
+#endif
+#ifdef F_SETNOSIGPIPE
+  DEBUG_VALUE_VALUE(f, cmd, F_SETNOSIGPIPE)
+#endif
+#ifdef F_GETNOSIGPIPE
+  DEBUG_VALUE_VALUE(f, cmd, F_GETNOSIGPIPE)
+#endif
+#ifdef F_TRANSCODEKEY
+  DEBUG_VALUE_VALUE(f, cmd, F_TRANSCODEKEY)
+#endif
+#ifdef F_SINGLE_WRITER
+  DEBUG_VALUE_VALUE(f, cmd, F_SINGLE_WRITER)
+#endif
+#ifdef F_GETPROTECTIONLEVEL
+  DEBUG_VALUE_VALUE(f, cmd, F_GETPROTECTIONLEVEL)
+#endif
+#ifdef F_FINDSIGS
+  DEBUG_VALUE_VALUE(f, cmd, F_FINDSIGS)
+#endif
+#ifdef F_ADDFILESIGS_FOR_DYLD_SIM
+  DEBUG_VALUE_VALUE(f, cmd, F_ADDFILESIGS_FOR_DYLD_SIM)
+#endif
+#ifdef F_BARRIERFSYNC
+  DEBUG_VALUE_VALUE(f, cmd, F_BARRIERFSYNC)
+#endif
+#ifdef F_OFD_SETLK
+  DEBUG_VALUE_VALUE(f, cmd, F_OFD_SETLK)
+#endif
+#ifdef F_OFD_SETLKW
+  DEBUG_VALUE_VALUE(f, cmd, F_OFD_SETLKW)
+#endif
+#ifdef F_OFD_GETLK
+  DEBUG_VALUE_VALUE(f, cmd, F_OFD_GETLK)
+#endif
+#ifdef F_OFD_SETLKWTIMEOUT
+  DEBUG_VALUE_VALUE(f, cmd, F_OFD_SETLKWTIMEOUT)
+#endif
+#ifdef F_ADDFILESIGS_RETURN
+  DEBUG_VALUE_VALUE(f, cmd, F_ADDFILESIGS_RETURN)
+#endif
+#ifdef F_CHECK_LV
+  DEBUG_VALUE_VALUE(f, cmd, F_CHECK_LV)
+#endif
+#ifdef F_PUNCHHOLE
+  DEBUG_VALUE_VALUE(f, cmd, F_PUNCHHOLE)
+#endif
+#ifdef F_TRIM_ACTIVE_FILE
+  DEBUG_VALUE_VALUE(f, cmd, F_TRIM_ACTIVE_FILE)
+#endif
+#ifdef F_SPECULATIVE_READ
+  DEBUG_VALUE_VALUE(f, cmd, F_SPECULATIVE_READ)
+#endif
+#ifdef F_GETPATH_NOFIRMLINK
+  DEBUG_VALUE_VALUE(f, cmd, F_GETPATH_NOFIRMLINK)
+#endif
+#ifdef F_ADDFILESIGS_INFO
+  DEBUG_VALUE_VALUE(f, cmd, F_ADDFILESIGS_INFO)
+#endif
+#ifdef F_ADDFILESUPPL
+  DEBUG_VALUE_VALUE(f, cmd, F_ADDFILESUPPL)
+#endif
+#ifdef F_GETSIGSINFO
+  DEBUG_VALUE_VALUE(f, cmd, F_GETSIGSINFO)
+#endif
+#ifdef F_TRANSFEREXTENTS
+  DEBUG_VALUE_VALUE(f, cmd, F_TRANSFEREXTENTS)
+#endif
+#ifdef F_ATTRIBUTION_TAG
+  DEBUG_VALUE_VALUE(f, cmd, F_ATTRIBUTION_TAG)
 #endif
   DEBUG_VALUE_END_DEC(f, cmd)
 }
@@ -255,6 +424,27 @@ void debug_fcntl_arg_or_ret(FILE *f, int cmd, int arg_or_ret) {
   }
 }
 
+/**
+ * Debug-print socket()'s 'domain' parameter.
+ */
+void debug_socket_domain(FILE *f, int domain) {
+  DEBUG_VALUE_START(f, domain)
+  DEBUG_VALUE_VALUE(f, domain, AF_UNIX)
+  DEBUG_VALUE_VALUE(f, domain, AF_INET)
+  DEBUG_VALUE_VALUE(f, domain, AF_APPLETALK)
+  DEBUG_VALUE_VALUE(f, domain, AF_INET6)
+#ifdef AF_KEY
+  DEBUG_VALUE_VALUE(f, domain, AF_KEY)
+#endif
+#ifdef AF_NETLINK
+  DEBUG_VALUE_VALUE(f, domain, AF_NETLINK)
+#endif
+#ifdef AF_PACKET
+  DEBUG_VALUE_VALUE(f, domain, AF_PACKET)
+#endif
+  DEBUG_VALUE_END_DEC(f, domain)
+}
+
 /**
  * Debug-print an error number.
  */
