@@ -20,6 +20,10 @@
 #define COMMON_PLATFORM_H_
 
 #include <assert.h>
+#ifdef __APPLE__
+#include <crt_externs.h>
+#define environ (*_NSGetEnviron())
+#endif
 #include <errno.h>
 #include <fcntl.h>
 #ifdef __linux__
@@ -119,7 +123,8 @@ struct statx {
 #define PRIssize "d"
 #endif
 
-#ifdef __APPLE__
+#if defined(__APPLE__) || (defined(_FILE_OFFSET_BITS) && (_FILE_OFFSET_BITS == 64) && \
+                           (__BITS_PER_LONG == 32))
 #define PRIoff "lld"
 #define SCNoff "lld"
 #else
@@ -139,6 +144,16 @@ struct statx {
 #define PRIloff PRIoff64
 #endif
 
+#if defined(__USE_TIME_BITS64) && defined(__BITS_PER_LONG)
+#if __BITS_PER_LONG == 32
+#define PRItime "lld"
+#else
+#define PRItime "ld"
+#endif
+#else
+#define PRItime "ld"
+#endif
+
 #ifdef __APPLE__
 #define sighandler_t sig_t
 #endif
