@@ -35,12 +35,11 @@
 #include "firebuild/forked_process.h"
 #include "firebuild/file_name.h"
 #include "firebuild/hash_cache.h"
+#include "firebuild/options.h"
 #include "firebuild/fbbfp.h"
 #include "firebuild/fbbstore.h"
 #include "firebuild/process_tree.h"
 
-extern bool generate_report;
-
 namespace firebuild {
 
 static const XXH64_hash_t kFingerprintVersion = 0;
@@ -360,17 +359,16 @@ bool ExecedProcessCacher::fingerprint(const ExecedProcess *proc) {
 
   add_to_hash_state(state, proc->libs().size());
   for (const auto lib : proc->libs()) {
-#ifdef __APPLE__
-    /* SDK libraries are not present as files, see:
-     * https://developer.apple.com/forums/thread/655588 */
-    if (strncmp(lib->c_str(), "/usr/lib/", strlen("/usr/lib/")) == 0) {
-      continue;
-    }
-#endif
     if (!hash_cache->get_hash(lib, 0, &hash)) {
+#ifdef __APPLE__
+      /* SDK libraries are not present as files, see:
+       * https://developer.apple.com/forums/thread/655588 */
+      hash = Hash();
+#else
       FB_DEBUG(FB_DEBUG_PROC, "Could not get hash of library: " + d(lib));
       maybe_XXH3_freeState(state);
       return false;
+#endif
     }
     add_to_hash_state(state, lib);
     add_to_hash_state(state, hash);
@@ -456,17 +454,16 @@ bool ExecedProcessCacher::fingerprint(const ExecedProcess *proc) {
     lib_builders.reserve(proc->libs().size());
 
     for (const auto& lib : proc->libs()) {
-#ifdef __APPLE__
-      /* SDK libraries are not present as files, see:
-       * https://developer.apple.com/forums/thread/655588 */
-      if (strncmp(lib->c_str(), "/usr/lib/", strlen("/usr/lib/")) == 0) {
-        continue;
-      }
-#endif
       if (!hash_cache->get_hash(lib, 0, &hash)) {
+#ifdef __APPLE__
+        /* SDK libraries are not present as files, see:
+         * https://developer.apple.com/forums/thread/655588 */
+        hash = Hash();
+#else
         FB_DEBUG(FB_DEBUG_PROC, "Could not get hash of library: " + d(lib));
         maybe_XXH3_freeState(state);
         return false;
+#endif
       }
       FBBFP_Builder_file& lib_builder = lib_builders.emplace_back();
       lib_builder.set_path(lib->c_str());
@@ -558,7 +555,7 @@ static bool dir_created_or_could_exist(
       /* Directory is expected to exist. */
       return true;
     }
-    parent_dir = FileName::GetParentDir(parent_dir->c_str(), parent_dir->length());
+    parent_dir = parent_dir->parent_dir();
   }
   return true;
 }
@@ -602,16 +599,21 @@ static bool tmp_file_or_on_tmp_path(const FileUsage* fu, const FileName* filenam
   }
 }
 
-static bool rustc_deps_dir(const ExecedProcess* const proc, const FileName* const filename) {
+static bool rustc_ignored_dir(const ExecedProcess* const proc, const FileName* const filename) {
   const std::vector<std::string> &args = proc->args();
-  if (args[0] == "rustc") {
-    for (const std::string& arg : args) {
+  if (args[0].ends_with("rustc")) {
+    for (size_t i = 1; i < args.size(); i++) {
+      const std::string& arg = args[i];
       if (arg.starts_with("dependency=")) {
         const std::string dependency_dir(arg.substr(strlen("dependency=")));
         /* Assumes that the dependency dir is already absolute. */
         if (dependency_dir == filename->to_string()) {
           return true;
         }
+      } else if (arg == "--out-dir" && args.size() > i + 1
+                 && args[i + 1] == filename->to_string()) {
+        /* Assumes that the output dir is already absolute. */
+        return true;
       }
     }
   }
@@ -708,12 +710,12 @@ void ExecedProcessCacher::store(ExecedProcess *proc) {
 
   /* Construct in_path_* in 2 passes. First collect the non-system paths and then the system paths,
    * for better performance. */
-  for (int i = 0; i < 2; i++) {
+  for (int pass = 0; pass < 2; pass++) {
     for (const auto& pair : proc->file_usages()) {
       const auto filename = pair.first;
       const FileUsage* fu = pair.second;
 
-      if (filename->is_in_read_only_location() == (i == 0)) {
+      if (filename->is_in_read_only_location() == (pass == 0)) {
         continue;
       }
 
@@ -722,7 +724,7 @@ void ExecedProcessCacher::store(ExecedProcess *proc) {
         FB_DEBUG(FB_DEBUG_CACHING,
                  "A file (" + d(filename)+ ") changed since the process used it.");
         proc->disable_shortcutting_only_this(
-            generate_report
+            Options::generate_report()
             ? deduplicated_string("A file (" + d(filename)
                                   + ") changed since the process used it.").c_str()
             : "A file could not be stored because it changed since the process used it.");
@@ -742,7 +744,7 @@ void ExecedProcessCacher::store(ExecedProcess *proc) {
         case ISDIR:
           if (fu->initial_state().hash_known()
               && ((quirks & FB_QUIRK_IGNORE_TMP_LISTING && filename == tmpdir)
-                  || rustc_deps_dir(proc, filename))) {
+                  || rustc_ignored_dir(proc, filename))) {
             FileInfo no_hash_initial_state(fu->initial_state());
             no_hash_initial_state.set_hash(nullptr);
             add_file(&in_path, filename, no_hash_initial_state);
@@ -760,11 +762,13 @@ void ExecedProcessCacher::store(ExecedProcess *proc) {
           break;
       }
     }
-    in_path_non_system_count = in_path.size();
-    in_path_notexist_non_system_count = in_path_notexist.size();
+    if (pass == 0) {
+      in_path_non_system_count = in_path.size();
+      in_path_notexist_non_system_count = in_path_notexist.size();
+    }
   }
 
-  uint64_t stored_blob_bytes = 0;
+  off_t stored_blob_bytes = 0;
   for (const auto& pair : proc->file_usages()) {
     const auto filename = pair.first;
     const FileUsage* fu = pair.second;
@@ -975,7 +979,8 @@ void ExecedProcessCacher::store(ExecedProcess *proc) {
    * locations. */
   struct {
     bool operator()(const FBBSTORE_Builder_file& a, const FBBSTORE_Builder_file& b) const {
-      return strcmp(a.get_path(), b.get_path()) < 0;
+      return memcmp(a.get_path(), b.get_path(),
+                    std::min(a.get_path_len(), b.get_path_len()) + 1) < 0;
     }
   } file_less;
   /* Sort non-system and system paths separately to not regress in shortcutting performance. */
@@ -986,7 +991,7 @@ void ExecedProcessCacher::store(ExecedProcess *proc) {
 
   struct {
     bool operator()(const cstring_view& a, const cstring_view& b) const {
-      return strcmp(a.c_str, b.c_str) < 0;
+      return memcmp(a.c_str, b.c_str, std::min(a.length, b.length) + 1) < 0;
     }
   } cstring_view_less;
   /* Sort non-system and system paths separately to not regress in shortcutting performance. */
@@ -1102,7 +1107,7 @@ static bool pio_matches_fs(const FBBSTORE_Serialized_process_inputs_outputs *can
     if (!hash_cache->file_info_matches(path, query)) {
       FB_DEBUG(FB_DEBUG_SHORTCUT, "│   " + d(subkey) + " mismatches e.g. at " + d(path));
       /* Store only the first mismatch. */
-      if (generate_report && !proc->shortcut_result()) {
+      if (Options::generate_report() && !proc->shortcut_result()) {
         proc->set_shortcut_result(deduplicated_string(
             d(subkey) + " mismatches e.g. at " + d(path)).c_str());
       }
@@ -1116,7 +1121,7 @@ static bool pio_matches_fs(const FBBSTORE_Serialized_process_inputs_outputs *can
     const FileInfo query(NOTEXIST);
     if (!hash_cache->file_info_matches(path, query)) {
       /* Store only the first mismatch. */
-      if (generate_report && !proc->shortcut_result()) {
+      if (Options::generate_report() && !proc->shortcut_result()) {
         proc->set_shortcut_result(deduplicated_string(
             d(subkey) + + " mismatches e.g. at " + d(path)
             +  ": path expected to be missing, existing object is found").c_str());
@@ -1146,7 +1151,7 @@ static bool pio_matches_fs(const FBBSTORE_Serialized_process_inputs_outputs *can
           /* The file has already been checked to be not writable and will be replaced while
            * applying the shortcut. */
         } else {
-          if (generate_report && !proc->shortcut_result()) {
+          if (Options::generate_report() && !proc->shortcut_result()) {
             proc->set_shortcut_result(deduplicated_string(
                 std::string("file to be written is not writable: ") + file->get_path()).c_str());
           }
@@ -1175,7 +1180,7 @@ const FBBSTORE_Serialized_process_inputs_outputs * ExecedProcessCacher::find_sho
   FB_DEBUG(FB_DEBUG_SHORTCUT, "│ Candidates:");
   const std::vector<Subkey> subkeys = obj_cache->list_subkeys(fingerprint);
   if (subkeys.empty()) {
-    if (generate_report) {
+    if (Options::generate_report()) {
       proc->set_shortcut_result(deduplicated_string("no candidate found").c_str());
     }
     FB_DEBUG(FB_DEBUG_SHORTCUT, "│   None found");
@@ -1190,7 +1195,7 @@ const FBBSTORE_Serialized_process_inputs_outputs * ExecedProcessCacher::find_sho
     }
     if (!obj_cache->retrieve(fingerprint, subkey.c_str(),
                              &candidate_inouts_buf, &candidate_inouts_buf_len)) {
-      if (generate_report) {
+      if (Options::generate_report()) {
         proc->set_shortcut_result(deduplicated_string(
             "could not retrieve " + d(subkey) + " from objcache").c_str());
       }
@@ -1358,24 +1363,6 @@ static void remove_files_and_dirs(
   }
 }
 
-static void close_all(std::vector<int>* fds) {
-  for (int fd : *fds) {
-    close(fd);
-  }
-}
-
-static bool add_blob_fd_from_hash(const XXH128_hash_t& fbb_hash, std::vector<int>* blob_fds) {
-  Hash hash(fbb_hash);
-  int fd;
-  if ((fd = blob_cache->get_fd_for_file(hash)) != -1) {
-    blob_fds->push_back(fd);
-  } else {
-    close_all(blob_fds);
-    return false;
-  }
-  return true;
-}
-
 /**
  * Applies the given shortcut.
  *
@@ -1388,7 +1375,24 @@ bool ExecedProcessCacher::apply_shortcut(ExecedProcess *proc,
   TRACK(FB_DEBUG_PROC, "proc=%s", D(proc));
 
   size_t i;
-  std::vector<int> blob_fds;
+  class BlobFds : public std::vector<int> {
+   public:
+    ~BlobFds() {
+      for (int fd : *this) {
+        close(fd);
+      }
+    }
+    bool add_from_hash(const XXH128_hash_t& fbb_hash) {
+      Hash hash(fbb_hash);
+      int fd;
+      if ((fd = blob_cache->get_fd_for_file(hash)) != -1) {
+        push_back(fd);
+        return true;
+      } else {
+        return false;
+      }
+    }
+  } blob_fds;
 
   const FBBSTORE_Serialized_process_outputs *outputs =
       reinterpret_cast<const FBBSTORE_Serialized_process_outputs *>
@@ -1400,7 +1404,7 @@ bool ExecedProcessCacher::apply_shortcut(ExecedProcess *proc,
     auto file = reinterpret_cast<const FBBSTORE_Serialized_file *>(outputs->get_path_isreg_at(i));
     if (file->get_type() == ISREG) {
       assert(file->has_hash());
-      if (!add_blob_fd_from_hash(file->get_hash(), &blob_fds)) {
+      if (!blob_fds.add_from_hash(file->get_hash())) {
         return false;
       }
     }
@@ -1408,13 +1412,13 @@ bool ExecedProcessCacher::apply_shortcut(ExecedProcess *proc,
   for (i = 0; i < outputs->get_append_to_fd_count(); i++) {
     auto append_to_fd = reinterpret_cast<const FBBSTORE_Serialized_append_to_fd *>
         (outputs->get_append_to_fd_at(i));
-    if (!add_blob_fd_from_hash(append_to_fd->get_hash(), &blob_fds)) {
+    if (!blob_fds.add_from_hash(append_to_fd->get_hash())) {
       return false;
     }
   }
 
   /* Bubble up all the file operations we're about to perform. */
-  ExecedProcess* registration_point = generate_report ? proc : proc->parent_exec_point();
+  ExecedProcess* registration_point = Options::generate_report() ? proc : proc->parent_exec_point();
   if (registration_point) {
     const FBBSTORE_Serialized_process_inputs *inputs =
         reinterpret_cast<const FBBSTORE_Serialized_process_inputs *>
@@ -1434,7 +1438,6 @@ bool ExecedProcessCacher::apply_shortcut(ExecedProcess *proc,
   }
 
   if (!restore_dirs(proc, outputs)) {
-    close_all(&blob_fds);
     return false;
   }
 
@@ -1488,16 +1491,17 @@ bool ExecedProcessCacher::apply_shortcut(ExecedProcess *proc,
   /* See what the process originally wrote to its inherited files (pipes or regular files).
    * Replay these. */
   for (i = 0; i < outputs->get_append_to_fd_count(); i++) {
-    auto append_to_fd = reinterpret_cast<const FBBSTORE_Serialized_append_to_fd *>
+    auto serialized_append_to_fd = reinterpret_cast<const FBBSTORE_Serialized_append_to_fd *>
         (outputs->get_append_to_fd_at(i));
-    FileFD *ffd = proc->get_fd(append_to_fd->get_fd());
+    const int append_to_fd = serialized_append_to_fd->get_fd();
+    FileFD *ffd = proc->get_fd(append_to_fd);
     assert(ffd);
 
     if (ffd->type() == FD_PIPE_OUT) {
       Pipe *pipe = ffd->pipe().get();
       assert(pipe);
 
-      Hash hash(append_to_fd->get_hash());
+      Hash hash(serialized_append_to_fd->get_hash());
       int fd = blob_fds[next_blob_fd_idx++];
       struct stat64 st;
       if (fstat64(fd, &st) < 0) {
@@ -1511,32 +1515,30 @@ bool ExecedProcessCacher::apply_shortcut(ExecedProcess *proc,
             pipe->proc2recorders[proc->parent_exec_point()];
         PipeRecorder::record_data_from_regular_fd(&recorders, fd, st.st_size);
       }
-      close(fd);
     } else if (ffd->type() == FD_FILE) {
       assert(ffd->filename());
 
       FB_DEBUG(FB_DEBUG_SHORTCUT,
                "│   Fetching file fragment from blobs cache: "
                + d(ffd->filename()));
-      Hash hash(append_to_fd->get_hash());
+      Hash hash(serialized_append_to_fd->get_hash());
       blob_cache->retrieve_file(blob_fds[next_blob_fd_idx++], ffd->filename(), true);
 
       /* Tell the interceptor to seek forward in this fd. */
-      fds_appended_to->push_back(ffd->fd());
+      fds_appended_to->push_back(append_to_fd);
     } else {
       assert(0 && "wrong file_fd type");
     }
 
     /* Bubble up the event that we wrote to this inherited fd. Currently this doesn't do anything,
      * but it might change in the future. */
-    proc->handle_write_to_inherited(ffd->fd(), false);
+    proc->handle_write_to_inherited(append_to_fd, false);
   }
 
   /* Set the exit code, propagate upwards. */
   // TODO(egmont) what to do with resource usage?
   proc->fork_point()->set_exit_status(outputs->get_exit_status());
 
-  close_all(&blob_fds);
   return true;
 }
 
@@ -1588,7 +1590,7 @@ bool ExecedProcessCacher::shortcut(ExecedProcess *proc, std::vector<int> *fds_ap
       if (inouts->has_cpu_time_ms()) {
         proc->add_shortcut_cpu_time_ms(inouts->get_cpu_time_ms());
       }
-    } else if (generate_report) {
+    } else if (Options::generate_report()) {
       proc->set_shortcut_result("applying shortcut failed");
     }
     /* Trigger cleanup of ProcessInputsOutputs. */
