@@ -22,6 +22,7 @@
 #include <tsl/hopscotch_map.h>
 #include <xxhash.h>
 
+#include <algorithm>
 #include <cstring>
 #include <string>
 #include <unordered_set>
@@ -50,6 +51,7 @@ class FileName {
   const char * c_str() const {return name_;}
   std::string to_string() const {return std::string(name_);}
   uint32_t length() const {return length_;}
+  const FileName* parent_dir() const {return GetParentDir(name_, length_);}
   size_t hash() const {return XXH3_64bits(name_, length_);}
   const XXH128_hash_t& hash_XXH128() const {
     auto it = hash_db_->find(this);
@@ -109,7 +111,7 @@ class FileName {
  private:
   FileName(const char * const name, size_t length, bool copy_name)
       : name_(copy_name ? reinterpret_cast<const char *>(malloc(length + 1)) : name),
-        length_(length), in_ignore_location_(false), in_read_only_location_(false) {
+        length_(length) {
     if (copy_name) {
       memcpy(const_cast<char*>(name_), name, length);
       const_cast<char*>(name_)[length] = '\0';
@@ -125,8 +127,8 @@ class FileName {
 
   const char * const name_;
   const uint32_t length_;
-  const bool in_ignore_location_;
-  const bool in_read_only_location_;
+  const bool in_ignore_location_ = false;
+  const bool in_read_only_location_ = false;
   static std::unordered_set<FileName, FileNameHasher>* db_;
   static tsl::hopscotch_map<const FileName*, XXH128_hash_t>* hash_db_;
   /** Number of FileOFDs open for writing referencing this file. */
@@ -166,7 +168,7 @@ struct FileNameHasher {
 /** Helper struct for std::sort */
 struct FileNameLess {
   bool operator()(const FileName* f1, const FileName* f2) const {
-    return strcmp(f1->c_str(), f2->c_str()) < 0;
+    return memcmp(f1->c_str(), f2->c_str(), std::min(f1->length(), f2->length()) + 1) < 0;
   }
 };
 
