@@ -19,11 +19,15 @@
 #include "firebuild/message_processor.h"
 
 #include <sys/random.h>
+#if defined (__APPLE__)
+#include <sys/spawn.h>
+#endif
 #include <sys/types.h>
 #include <sys/wait.h>
 
 #include <memory>
 #include <string>
+#include <unordered_set>
 #include <utility>
 #include <vector>
 
@@ -167,6 +171,11 @@ void MessageProcessor::accept_exec_child(ExecedProcess* proc, int fd_conn,
           auto file_fd_old = proc->get_shared_fd(inherited_file.fds[0]);
           auto pipe = file_fd_old->pipe();
           assert(pipe);
+          if (pipe->finished()) {
+            /* Pipe may have been closed (or broken) before the supervisor started accepting
+             * the new child*/
+            continue;
+          }
 
           /* As per #689, reopening the pipes causes different behavior than without firebuild. With
            * firebuild, across an exec they no longer share the same "open file description" and thus
@@ -174,12 +183,12 @@ void MessageProcessor::accept_exec_child(ExecedProcess* proc, int fd_conn,
            * point to a new FileOFD. */
           auto fds = proc->fds();
           int fd = inherited_file.fds[0];
-          auto file_fd = std::make_shared<FileFD>(fd, file_fd_old->flags(), pipe,
+          auto file_fd = std::make_shared<FileFD>(file_fd_old->flags(), pipe,
                                                   file_fd_old->opened_by());
           (*fds)[fd] = file_fd;
           for (size_t i = 1; i < inherited_file.fds.size(); i++) {
             fd = inherited_file.fds[i];
-            auto file_fd_dup = std::make_shared<FileFD>(fd, file_fd, false);
+            auto file_fd_dup = std::make_shared<FileFD>(file_fd, false);
             (*fds)[fd] = file_fd_dup;
           }
 
@@ -301,8 +310,7 @@ static void accept_popen_child(Process* unix_parent, const pending_popen_t *pend
 #endif
 
     /* The reading side of this pipe is in the popening (parent) process. */
-    auto ffd0 = std::make_shared<FileFD>(pending_popen->fd /* client fd */,
-                                         (flags & ~O_ACCMODE) | O_RDONLY,
+    auto ffd0 = std::make_shared<FileFD>((flags & ~O_ACCMODE) | O_RDONLY,
                                          pipe->fd0_shared_ptr(),
                                          unix_parent /* creator */,
                                          true /* close_on_popen */);
@@ -310,8 +318,7 @@ static void accept_popen_child(Process* unix_parent, const pending_popen_t *pend
 
     /* The writing side of this pipe is in the forked and the execed processes.
      * We're lazy and we don't register it for the forked process, no one cares. */
-    auto ffd1 = std::make_shared<FileFD>(STDOUT_FILENO /* client fd */,
-                                         (flags & ~O_ACCMODE) | O_WRONLY,
+    auto ffd1 = std::make_shared<FileFD>((flags & ~O_ACCMODE) | O_WRONLY,
                                          pipe->fd1_shared_ptr(),
                                          unix_parent /* creator */,
                                          false /* close_on_popen */);
@@ -354,16 +361,14 @@ static void accept_popen_child(Process* unix_parent, const pending_popen_t *pend
 
     /* The reading side of this pipe is in the forked and the execed processes.
      * We're lazy and we don't register it for the forked process, no one cares. */
-    auto ffd0 = std::make_shared<FileFD>(STDIN_FILENO /* client fd */,
-                                         (flags & ~O_ACCMODE) | O_RDONLY,
+    auto ffd0 = std::make_shared<FileFD>((flags & ~O_ACCMODE) | O_RDONLY,
                                          pipe->fd0_shared_ptr(),
                                          unix_parent /* creator */,
                                          false /* close_on_popen */);
     proc->add_filefd(STDIN_FILENO /* client fd */, ffd0);
 
     /* The (so far only) writing side of this pipe is in the popening (parent) process. */
-    auto ffd1 = std::make_shared<FileFD>(pending_popen->fd /* client fd */,
-                                         (flags & ~O_ACCMODE) | O_WRONLY,
+    auto ffd1 = std::make_shared<FileFD>((flags & ~O_ACCMODE) | O_WRONLY,
                                          pipe->fd1_shared_ptr(),
                                          unix_parent /* creator */,
                                          true /* close_on_popen */);
@@ -407,10 +412,10 @@ static void accept_fork_child(Process* parent, int parent_fd, int parent_ack,
 
 /**
  * Process message coming from interceptor
- * @param fb_conn file desctiptor of the connection
+ * @param fb_conn file descriptor of the connection
  */
 static void proc_new_process_msg(const FBBCOMM_Serialized *fbbcomm_buf, uint16_t ack_id,
-                          int fd_conn, Process** new_proc) {
+                                 int fd_conn, Process** new_proc) {
   TRACK(FB_DEBUG_PROC, "fd_conn=%s, ack_id=%d", D_FD(fd_conn), ack_id);
 
   int tag = fbbcomm_buf->get_tag();
@@ -511,9 +516,11 @@ static void proc_new_process_msg(const FBBCOMM_Serialized *fbbcomm_buf, uint16_t
          *  POSIX states popen shall ensure that any streams from previous popen()
          *  calls that remain open in the parent process should be closed in the new
          *  child process. [...] */
-        for (auto& file_fd : *parent->fds()) {
+        int fds_size = parent->fds()->size();
+        for (int fd = 0; fd < fds_size; fd++) {
+          const FileFD* file_fd = parent->get_fd(fd);
           if (file_fd && file_fd->close_on_popen()) {
-            parent->handle_close(file_fd->fd());
+            parent->handle_close(fd);
           }
         }
       }
@@ -580,6 +587,152 @@ static void proc_new_process_msg(const FBBCOMM_Serialized *fbbcomm_buf, uint16_t
   }
 }
 
+static void posix_spawn_preopen_files(const FBBCOMM_Serialized_posix_spawn * const ic_msg,
+                                      Process* const proc) {
+  for (size_t i = 0; i < ic_msg->get_file_actions_count(); i++) {
+    const FBBCOMM_Serialized *action = ic_msg->get_file_actions_at(i);
+    switch (action->get_tag()) {
+      case FBBCOMM_TAG_posix_spawn_file_action_open: {
+        /* A successful open to a particular fd, silently closing the previous file if any. */
+        const FBBCOMM_Serialized_posix_spawn_file_action_open *action_open =
+            reinterpret_cast<const FBBCOMM_Serialized_posix_spawn_file_action_open *>(action);
+        int flags = action_open->get_flags();
+        if (is_write(flags)) {
+          const FileName* file_name = proc->get_absolute(
+              AT_FDCWD, action_open->get_pathname(), action_open->get_pathname_len());
+          if (file_name) {
+            /* Pretend that the parent opened the file for writing and not the fork child.
+             * This is not accurate, but the fork child does not exist yet. A parallel
+             * process opening the file for writing would disable shortcutting the same way. */
+            file_name->open_for_writing(proc->exec_point());
+          }
+        }
+        break;
+      }
+      default:
+        /* Only opens are handled (as pre_opens). */
+        break;
+    }
+  }
+}
+
+template<class P>
+static void process_posix_spawn_file_actions(const P * const ic_msg, Process* const proc) {
+#ifdef __APPLE__
+      std::unordered_set<int> new_fds {};
+      const int attr_flags = ic_msg->get_attr_flags_with_fallback(0);
+#endif
+      for (size_t i = 0; i < ic_msg->get_file_actions_count(); i++) {
+        const FBBCOMM_Serialized *action = ic_msg->get_file_actions_at(i);
+        switch (action->get_tag()) {
+          case FBBCOMM_TAG_posix_spawn_file_action_open: {
+            /* A successful open to a particular fd, silently closing the previous file if any. */
+            auto action_open =
+                reinterpret_cast<const FBBCOMM_Serialized_posix_spawn_file_action_open *>(action);
+            const char *pathname = action_open->get_pathname();
+            const size_t pathname_len = action_open->get_pathname_len();
+            int fd = action_open->get_fd();
+#ifdef __APPLE__
+            if (attr_flags & POSIX_SPAWN_CLOEXEC_DEFAULT) {
+              new_fds.insert(fd);
+            }
+#endif
+            int flags = action_open->get_flags();
+            mode_t mode = action_open->get_mode();
+            proc->handle_force_close(fd);
+            proc->handle_open(AT_FDCWD, pathname, pathname_len, flags, mode, fd, 0, -1, 0,
+                              is_write(flags), false);
+            break;
+          }
+          case FBBCOMM_TAG_posix_spawn_file_action_close: {
+            /* A close attempt, maybe successful, maybe failed, we don't know. See glibc's
+             * sysdeps/unix/sysv/linux/spawni.c:
+             *   Signal errors only for file descriptors out of range.
+             * sysdeps/posix/spawni.c:
+             *   Only signal errors for file descriptors out of range.
+             * whereas signaling the error means to abort posix_spawn and thus not reach
+             * this code here. */
+            auto action_close =
+                reinterpret_cast<const FBBCOMM_Serialized_posix_spawn_file_action_close *>(action);
+            int fd = action_close->get_fd();
+            proc->handle_force_close(fd);
+            break;
+          }
+          case FBBCOMM_TAG_posix_spawn_file_action_closefrom: {
+            /* A successful closefrom. */
+            auto action_closefrom =
+                reinterpret_cast<const FBBCOMM_Serialized_posix_spawn_file_action_closefrom *>
+                (action);
+            int lowfd = action_closefrom->get_lowfd();
+            proc->handle_closefrom(lowfd);
+            break;
+          }
+          case FBBCOMM_TAG_posix_spawn_file_action_dup2: {
+            /* A successful dup2.
+             * Note that as per https://austingroupbugs.net/view.php?id=411 and glibc's
+             * implementation, oldfd==newfd clears the close-on-exec bit (here only,
+             * not in a real dup2()). */
+            auto action_dup2 =
+                reinterpret_cast<const FBBCOMM_Serialized_posix_spawn_file_action_dup2 *>(action);
+            int oldfd = action_dup2->get_oldfd();
+            int newfd = action_dup2->get_newfd();
+            if (oldfd == newfd) {
+              proc->handle_clear_cloexec(oldfd);
+            } else {
+              proc->handle_dup3(oldfd, newfd, 0, 0);
+            }
+#ifdef __APPLE__
+            if (attr_flags & POSIX_SPAWN_CLOEXEC_DEFAULT) {
+              new_fds.insert(newfd);
+            }
+#endif
+            break;
+          }
+          case FBBCOMM_TAG_posix_spawn_file_action_chdir: {
+            /* A successful chdir. */
+            auto action_chdir =
+                reinterpret_cast<const FBBCOMM_Serialized_posix_spawn_file_action_chdir *>(action);
+            const char *pathname = action_chdir->get_pathname();
+            proc->handle_set_wd(pathname);
+            break;
+          }
+          case FBBCOMM_TAG_posix_spawn_file_action_fchdir: {
+            /* A successful fchdir. */
+            auto action_fchdir =
+                reinterpret_cast<const FBBCOMM_Serialized_posix_spawn_file_action_fchdir *>(action);
+            int fd = action_fchdir->get_fd();
+            proc->handle_set_fwd(fd);
+            break;
+          }
+#ifdef __APPLE__
+          case FBBCOMM_TAG_posix_spawn_file_action_inherit: {
+            /* A successful inherit. */
+            auto action_inherit =
+                reinterpret_cast<const FBBCOMM_Serialized_posix_spawn_file_action_inherit *>(
+                    action);
+            int fd = action_inherit->get_fd();
+            if (attr_flags & POSIX_SPAWN_CLOEXEC_DEFAULT) {
+              new_fds.insert(fd);
+            }
+            proc->handle_clear_cloexec(fd);
+            break;
+          }
+#endif
+          default:
+            assert(false);
+        }
+      }
+#ifdef __APPLE__
+      if (attr_flags & POSIX_SPAWN_CLOEXEC_DEFAULT) {
+        for (size_t fd = 0; fd < proc->fds()->size(); fd++) {
+          if (!new_fds.contains(fd)) {
+            proc->handle_force_close(fd);
+          }
+        }
+      }
+#endif
+}
+
 static void proc_ic_msg(const FBBCOMM_Serialized *fbbcomm_buf, uint16_t ack_num,
                         int fd_conn, Process* proc) {
   TRACKX(FB_DEBUG_COMM, 1, 1, Process, proc, "fd_conn=%s, tag=%s, ack_num=%d",
@@ -621,13 +774,24 @@ static void proc_ic_msg(const FBBCOMM_Serialized *fbbcomm_buf, uint16_t ack_num,
       auto expected_child = new ExecedProcessEnv(proc->pass_on_fds(false), LAUNCH_TYPE_SYSTEM);
       if (ic_msg->has_cmd()) {
         expected_child->set_sh_c_command(ic_msg->get_cmd());
+#ifdef __APPLE__
+      } else {
+        /* OS X's system(NULL) just checks /bin/sh instead of running it. */
+        break;
+#endif
       }
       proc->set_expected_child(expected_child);
       break;
     }
     case FBBCOMM_TAG_system_ret: {
-      assert(proc->system_child());
       auto ic_msg = reinterpret_cast<const FBBCOMM_Serialized_system_ret *>(fbbcomm_buf);
+#ifdef __APPLE__
+      /* OS X's system(NULL) just checks /bin/sh instead of running it. */
+      if (!ic_msg->has_cmd()) {
+        break;
+      }
+#endif
+      assert(proc->system_child());
       /* system() implicitly waits for the child to finish. */
       int ret = ic_msg->get_ret();
       if (ret == -1 || !WIFEXITED(ret)) {
@@ -726,6 +890,16 @@ static void proc_ic_msg(const FBBCOMM_Serialized *fbbcomm_buf, uint16_t ack_num,
     }
     case FBBCOMM_TAG_posix_spawn: {
       auto ic_msg = reinterpret_cast<const FBBCOMM_Serialized_posix_spawn *>(fbbcomm_buf);
+#ifdef __APPLE__
+      const int attr_flags = ic_msg->get_attr_flags_with_fallback(0);
+      if (attr_flags & POSIX_SPAWN_SETEXEC) {
+        proc->update_rusage(ic_msg->get_utime_u(), ic_msg->get_stime_u());
+        // FIXME(rbalint) save parameters of pending exec()-ed process
+        process_posix_spawn_file_actions<FBBCOMM_Serialized_posix_spawn>(ic_msg, proc);
+        proc->set_exec_pending(true);
+        break;
+      }
+#endif
       auto expected_child = new ExecedProcessEnv(proc->pass_on_fds(false), LAUNCH_TYPE_POSIX_SPAWN);
       std::vector<std::string> argv = ic_msg->get_arg_as_vector();
       expected_child->set_argv(argv);
@@ -733,31 +907,7 @@ static void proc_ic_msg(const FBBCOMM_Serialized *fbbcomm_buf, uint16_t ack_num,
       proc->set_posix_spawn_pending(true);
       /* The actual forked process might perform some file operations according to
        * posix_spawn()'s file_actions. Pre-open the files to be written. */
-      for (size_t i = 0; i < ic_msg->get_file_actions_count(); i++) {
-        const FBBCOMM_Serialized *action = ic_msg->get_file_actions_at(i);
-        switch (action->get_tag()) {
-          case FBBCOMM_TAG_posix_spawn_file_action_open: {
-            /* A successful open to a particular fd, silently closing the previous file if any. */
-            const FBBCOMM_Serialized_posix_spawn_file_action_open *action_open =
-                reinterpret_cast<const FBBCOMM_Serialized_posix_spawn_file_action_open *>(action);
-            int flags = action_open->get_flags();
-            if (is_write(flags)) {
-              const FileName* file_name = proc->get_absolute(
-                  AT_FDCWD, action_open->get_pathname(), action_open->get_pathname_len());
-              if (file_name) {
-                /* Pretend that the parent opened the file for writing and not the fork child.
-                 * This is not accurate, but the fork child does not exist yet. A parallel
-                 * process opening the file for writing would disable shortcutting the same way. */
-                file_name->open_for_writing(proc->exec_point());
-              }
-            }
-            break;
-          }
-          default:
-            /* Only opens are handled (as pre_opens). */
-            break;
-        }
-      }
+      posix_spawn_preopen_files(ic_msg, proc);
       break;
     }
     case FBBCOMM_TAG_posix_spawn_parent: {
@@ -769,91 +919,7 @@ static void proc_ic_msg(const FBBCOMM_Serialized *fbbcomm_buf, uint16_t ack_num,
 
       /* The actual forked process might perform some file operations according to
        * posix_spawn()'s file_actions. Do the corresponding administration. */
-      for (size_t i = 0; i < ic_msg->get_file_actions_count(); i++) {
-        const FBBCOMM_Serialized *action = ic_msg->get_file_actions_at(i);
-        switch (action->get_tag()) {
-          case FBBCOMM_TAG_posix_spawn_file_action_open: {
-            /* A successful open to a particular fd, silently closing the previous file if any. */
-            auto action_open =
-                reinterpret_cast<const FBBCOMM_Serialized_posix_spawn_file_action_open *>(action);
-            const char *pathname = action_open->get_pathname();
-            const size_t pathname_len = action_open->get_pathname_len();
-            int fd = action_open->get_fd();
-            int flags = action_open->get_flags();
-            mode_t mode = action_open->get_mode();
-            fork_child->handle_force_close(fd);
-            fork_child->handle_open(AT_FDCWD, pathname, pathname_len, flags, mode, fd, 0, -1, 0,
-                                    false, false);
-            /* Revert the effect of "pre-opening" paths to be written in the posix_spawn message.*/
-            if (is_write(flags)) {
-              const FileName* file_name = fork_child->get_absolute(AT_FDCWD, pathname,
-                                                                              pathname_len);
-              if (file_name) {
-                file_name->close_for_writing();
-              }
-            }
-            break;
-          }
-          case FBBCOMM_TAG_posix_spawn_file_action_close: {
-            /* A close attempt, maybe successful, maybe failed, we don't know. See glibc's
-             * sysdeps/unix/sysv/linux/spawni.c:
-             *   Signal errors only for file descriptors out of range.
-             * sysdeps/posix/spawni.c:
-             *   Only signal errors for file descriptors out of range.
-             * whereas signaling the error means to abort posix_spawn and thus not reach
-             * this code here. */
-            auto action_close =
-                reinterpret_cast<const FBBCOMM_Serialized_posix_spawn_file_action_close *>(action);
-            int fd = action_close->get_fd();
-            fork_child->handle_force_close(fd);
-            break;
-          }
-          case FBBCOMM_TAG_posix_spawn_file_action_closefrom: {
-            /* A successful closefrom. */
-            auto action_closefrom =
-                reinterpret_cast<const FBBCOMM_Serialized_posix_spawn_file_action_closefrom *>
-                (action);
-            int lowfd = action_closefrom->get_lowfd();
-            fork_child->handle_closefrom(lowfd);
-            break;
-          }
-          case FBBCOMM_TAG_posix_spawn_file_action_dup2: {
-            /* A successful dup2.
-             * Note that as per https://austingroupbugs.net/view.php?id=411 and glibc's
-             * implementation, oldfd==newfd clears the close-on-exec bit (here only,
-             * not in a real dup2()). */
-            auto action_dup2 =
-                reinterpret_cast<const FBBCOMM_Serialized_posix_spawn_file_action_dup2 *>(action);
-            int oldfd = action_dup2->get_oldfd();
-            int newfd = action_dup2->get_newfd();
-            if (oldfd == newfd) {
-              fork_child->handle_clear_cloexec(oldfd);
-            } else {
-              fork_child->handle_dup3(oldfd, newfd, 0, 0);
-            }
-            break;
-          }
-          case FBBCOMM_TAG_posix_spawn_file_action_chdir: {
-            /* A successful chdir. */
-            auto action_chdir =
-                reinterpret_cast<const FBBCOMM_Serialized_posix_spawn_file_action_chdir *>(action);
-            const char *pathname = action_chdir->get_pathname();
-            fork_child->handle_set_wd(pathname);
-            break;
-          }
-          case FBBCOMM_TAG_posix_spawn_file_action_fchdir: {
-            /* A successful fchdir. */
-            auto action_fchdir =
-                reinterpret_cast<const FBBCOMM_Serialized_posix_spawn_file_action_fchdir *>(action);
-            int fd = action_fchdir->get_fd();
-            fork_child->handle_set_fwd(fd);
-            break;
-          }
-          default:
-            assert(false);
-        }
-      }
-
+      process_posix_spawn_file_actions<FBBCOMM_Serialized_posix_spawn_parent>(ic_msg, fork_child);
       proc->set_posix_spawn_pending(false);
 
       auto posix_spawn_child_sock = proc_tree->Pid2PosixSpawnChildSock(proc->pid());
@@ -1004,6 +1070,10 @@ static void proc_ic_msg(const FBBCOMM_Serialized *fbbcomm_buf, uint16_t ack_num,
       PFBBA_HANDLE(proc, close_range, fbbcomm_buf);
       break;
     }
+    case FBBCOMM_TAG_scandirat: {
+      PFBBA_HANDLE(proc, scandirat, fbbcomm_buf);
+      break;
+    }
     case FBBCOMM_TAG_truncate: {
       PFBBA_HANDLE(proc, truncate, fbbcomm_buf);
       break;
@@ -1092,6 +1162,16 @@ static void proc_ic_msg(const FBBCOMM_Serialized *fbbcomm_buf, uint16_t ack_num,
       PFBBA_HANDLE(proc, fchmodat, fbbcomm_buf);
       break;
     }
+    case FBBCOMM_TAG_shm_open: {
+      PFBBA_HANDLE(proc, shm_open, fbbcomm_buf);
+      break;
+    }
+#ifdef __APPLE__
+    case FBBCOMM_TAG_kqueue: {
+      PFBBA_HANDLE(proc, kqueue, fbbcomm_buf);
+      break;
+    }
+#endif
 #ifdef __linux__
     case FBBCOMM_TAG_memfd_create: {
       PFBBA_HANDLE(proc, memfd_create, fbbcomm_buf);
@@ -1181,10 +1261,18 @@ static void proc_ic_msg(const FBBCOMM_Serialized *fbbcomm_buf, uint16_t ack_num,
       PFBBA_HANDLE(proc, socketpair, fbbcomm_buf);
       break;
     }
+    case FBBCOMM_TAG_connect: {
+      PFBBA_HANDLE(proc, connect, fbbcomm_buf);
+      break;
+    }
     case FBBCOMM_TAG_statfs:
       ProcessFBBAdaptor::handle(
            proc, reinterpret_cast<const FBBCOMM_Serialized_statfs *>(fbbcomm_buf));
       break;
+    case FBBCOMM_TAG_mktemp:
+      ProcessFBBAdaptor::handle(
+           proc, reinterpret_cast<const FBBCOMM_Serialized_mktemp *>(fbbcomm_buf));
+      break;
     case FBBCOMM_TAG_gethostname:
       /* Ignore gethostname. With a local cache it should not make a difference, while
        * in a shared cache the intention is to use cached results from other machines. */
@@ -1193,6 +1281,10 @@ static void proc_ic_msg(const FBBCOMM_Serialized *fbbcomm_buf, uint16_t ack_num,
       /* Ignore seccomp(). The interposer always returns EINVAL error to keep interception
        * working. This breaks sandboxing, but builds can run arbitrary commands anyway. */
       break;
+    case FBBCOMM_TAG_mac_syscall:
+      /* Ignore __mac_syscall(). Having the tag defined allows easier debugging with -d comm. */
+      // TODO(rbalint) check if any __mac_syscall() invocation could impact builds
+      break;
     case FBBCOMM_TAG_fb_debug:
     case FBBCOMM_TAG_fb_error:
     case FBBCOMM_TAG_fchownat:
