@@ -36,6 +36,7 @@
 #include <sys/types.h>
 #include <unistd.h>
 
+#include <algorithm>
 #include <cinttypes>
 #include <utility>
 
@@ -44,6 +45,7 @@
 #include "firebuild/pipe_recorder.h"
 #include "firebuild/config.h"
 #include "firebuild/execed_process.h"
+#include "firebuild/hash_cache.h"
 #include "firebuild/forked_process.h"
 #include "firebuild/execed_process_env.h"
 #include "firebuild/process_tree.h"
@@ -57,10 +59,10 @@ static int fb_pid_counter;
 Process::Process(const int pid, const int ppid, const int exec_count, const FileName *wd,
                  const mode_t umask, Process * parent, std::vector<std::shared_ptr<FileFD>>* fds,
                  const bool debug_suppressed)
-    : parent_(parent), state_(FB_PROC_RUNNING),
+    : parent_(parent),
       fb_pid_(fb_pid_counter++), pid_(pid), ppid_(ppid), exec_count_(exec_count), wd_(wd),
       umask_(umask), fds_(fds), fork_children_(), expected_child_(),
-      debug_suppressed_(debug_suppressed), exec_child_(NULL) {
+      debug_suppressed_(debug_suppressed) {
   TRACKX(FB_DEBUG_PROC, 0, 1, Process, this, "pid=%d, ppid=%d, parent=%s", pid, ppid, D(parent));
 }
 
@@ -156,7 +158,11 @@ std::vector<std::shared_ptr<FileFD>>* Process::pass_on_fds(const bool execed) co
   for (int i = 0; i < fds_size; i++) {
     const FileFD* const raw_file_fd = (*fds_)[i].get();
     if (raw_file_fd != nullptr) {
-      if (!(execed && raw_file_fd->cloexec())) {
+      if (!(execed && raw_file_fd->cloexec())
+#ifdef __APPLE__
+          && !(!execed && raw_file_fd->close_on_fork())
+#endif
+          ) {
         /* The operations on the fds in the new process don't affect the fds in the parent,
          * thus create a copy of the parent's FileFD pointed to by a new shared pointer. */
         (*ret_fds)[i] = std::make_shared<FileFD>(*raw_file_fd);
@@ -227,9 +233,9 @@ int Process::handle_open(const int dirfd, const char * const ar_name, const size
       /* open(..., O_TMPFILE) does not really open a file, but creates an unnamed temporary file in
        * the specified directory. Treat it like it was a memory backed file and later register the
        * directory's usage. */
-      add_filefd(fd, std::make_shared<FileFD>(fd, flags, FD_SPECIAL, this));
+      add_filefd(fd, std::make_shared<FileFD>(flags, FD_SPECIAL, this));
     } else {
-      add_filefd(fd, std::make_shared<FileFD>(name, fd, flags, this));
+      add_filefd(fd, std::make_shared<FileFD>(name, flags, this));
     }
   }
 
@@ -320,14 +326,14 @@ int Process::handle_force_close(const int fd) {
   return 0;
 }
 
-void Process::handle_close(FileFD * file_fd) {
+void Process::handle_close(FileFD * file_fd, const int fd) {
   auto pipe = file_fd->pipe().get();
   if (pipe) {
     /* There may be data pending, drain it and register closure. */
     pipe->handle_close(file_fd);
     file_fd->set_pipe(nullptr);
   }
-  (*fds_)[file_fd->fd()].reset();
+  (*fds_)[fd].reset();
 }
 
 int Process::handle_close(const int fd, const int error) {
@@ -359,7 +365,7 @@ int Process::handle_close(const int fd, const int error) {
         return -1;
       }
     } else {
-      handle_close(file_fd);
+      handle_close(file_fd, fd);
       return 0;
     }
   }
@@ -379,18 +385,18 @@ int Process::handle_close_range(const unsigned int first, const unsigned int las
   (void)flags;  /* might be unused */
 
   if (!error) {
-    for (auto& file_fd : *fds_) {
+    unsigned int fds_size = fds_->size();
+    const int range_max = std::min(last, fds_size - 1);
+    for (int fd = first; fd <= range_max; fd++) {
+      FileFD* file_fd = (*fds_)[fd].get();
       if (!file_fd) {
         continue;
       }
-      unsigned int fd = static_cast<unsigned int>(file_fd->fd());
-      if (fd >= first && fd <= last) {
-        if (flags & CLOSE_RANGE_CLOEXEC) {
-          /* Don't close, just set the cloexec bit. */
-          file_fd->set_cloexec(true);
-        } else {
-          handle_close(file_fd.get());
-        }
+      if (flags & CLOSE_RANGE_CLOEXEC) {
+        /* Don't close, just set the cloexec bit. */
+        file_fd->set_cloexec(true);
+      } else {
+        handle_close(fd);
       }
     }
   }
@@ -398,32 +404,95 @@ int Process::handle_close_range(const unsigned int first, const unsigned int las
 }
 
 
-int Process::handle_dlopen(const char * const absolute_filename,
-                           const size_t absolute_filename_len,
+int Process::handle_dlopen(const std::vector<std::string>& libs,
                            const char * const looked_up_filename,
                            const size_t looked_up_filename_len,
                            const bool error, int fd_conn, int ack_num) {
-  if (absolute_filename) {
+  int ret = 0;
+  if (libs.size() > 0) {
+    /* There are new loaded shared libraries, register them. */
+    for (const std::string& ar_name : libs) {
+      const FileName* name = get_absolute(AT_FDCWD, ar_name.c_str(), ar_name.size());
+#ifdef __APPLE__
+      if (!hash_cache->get_statinfo(name, nullptr, nullptr)) {
+        /* Some libraries are not present as regular files on macOS, ignore those. */
+        continue;
+      }
+#endif
+      FileUsageUpdate update = FileUsageUpdate::get_from_open_params(name, O_RDONLY, 0, 0, false);
+      if (!exec_point()->register_file_usage_update(name, update)) {
+        exec_point()->disable_shortcutting_bubble_up(
+            "Could not register loading the shared library", *name);
+      }
+    }
+  }
+  if (error) {
     /* When failing to dlopen() a file assume it is not present.
      * This is a safe assumption for shortcutting purposes, since the cache entry
      * will require the the file to be missing to shortcut the process and if the file
      * is missing dlopen() would have failed for sure.
      */
-    return handle_open(AT_FDCWD, absolute_filename, absolute_filename_len,
-                             O_RDONLY, 0, -1, error ? ENOENT : 0, fd_conn, ack_num, false, false);
-  } else if (looked_up_filename) {
-    /* Failed dlopen() could not find the file on the search path.*/
-    /* TODO(rbalint) allow shortcutting the process and mark the file as missing on all the
-     * search path entries as described in dlopen(3). */
-    (void)looked_up_filename_len;
-    exec_point()->disable_shortcutting_bubble_up("Process failed to dlopen() ", looked_up_filename);
-    return -1;
-  } else {
-    if (error) {
-      exec_point()->disable_shortcutting_bubble_up("Process failed to dlopen() the main program");
+    if (strchr(looked_up_filename, '/')) {
+      /* Failed to dlopen() a relative or absolute filename, register the file as missing. */
+      handle_open(AT_FDCWD, looked_up_filename, looked_up_filename_len,
+                  O_RDONLY, 0, -1, ENOENT, 0, 0, false, false);
+    } else {
+      /* Failed dlopen() could not find the file on the search path. */
+      /* TODO(rbalint) allow shortcutting the process and mark the file as missing on all the
+       * search path entries as described in dlopen(3). */
+      exec_point()->disable_shortcutting_bubble_up("Process failed to dlopen() ",
+                                                   looked_up_filename);
+      ret = -1;
+    }
+  }
+  // TODO(rbalint) maybe move ACK-ing earlier, before the bubble-ups
+  if (ack_num != 0) {
+    ack_msg(fd_conn, ack_num);
+  }
+  return ret;
+}
+
+int Process::handle_scandirat(const int dirfd, const char * const ar_name, const size_t ar_len,
+                              const int error) {
+  TRACKX(FB_DEBUG_PROC, 1, 1, Process, this,
+         "dirfd=%d, ar_name=%s, error=%d", dirfd, D(ar_name), error);
+
+  if (error) {
+    if (error == ENOENT) {
+      /* scandirat() failed, because the directory doesn't exist. */
+      const FileName* name = get_absolute(dirfd, ar_name, ar_len);
+      if (!name) {
+        exec_point()->disable_shortcutting_bubble_up(
+            "Invalid dirfd or filename passed to scandirat()");
+        return -1;
+      }
+      FileUsageUpdate update(name, NOTEXIST);
+      if (!exec_point()->register_file_usage_update(name, update)) {
+        exec_point()->disable_shortcutting_bubble_up("Could not register scandirat() on", *name);
+        return -1;
+      }
+      return 0;
+    } else {
+      /* scandirat() failed, but we don't know why. */
+      exec_point()->disable_shortcutting_bubble_up("scandirat() failed");
       return -1;
     }
   }
+
+  const FileName* name = get_absolute(dirfd, ar_name, ar_len);
+  if (!name) {
+    exec_point()->disable_shortcutting_bubble_up(
+        "Invalid dirfd or filename passed to scandirat()");
+    return -1;
+  }
+
+  FileUsageUpdate update =
+       FileUsageUpdate::get_from_open_params(name, O_RDONLY | O_DIRECTORY, 0, error, false);
+  if (!exec_point()->register_file_usage_update(name, update)) {
+    exec_point()->disable_shortcutting_bubble_up("Could not register scandirat() on", *name);
+    return -1;
+  }
+
   return 0;
 }
 
@@ -579,6 +648,22 @@ int Process::handle_statfs(const char * const a_name, const size_t length,
   return 0;
 }
 
+int Process::handle_mktemp(const char * const a_name, const size_t length) {
+  TRACKX(FB_DEBUG_PROC, 1, 1, Process, this, "a_name=%s", D(a_name));
+
+  /* Operating on a file reached by its name, made absolute by the interceptor. */
+#ifdef FB_EXTRA_DEBUG
+  assert(path_is_absolute(a_name));
+#endif
+  const FileName* name = FileName::Get(a_name, length);
+  /* The only thing the process knows about that file is that it does not exist. */
+  FileUsageUpdate update = FileUsageUpdate::get_from_open_params(name, O_RDWR, 0, ENOENT, true);
+  if (!exec_point()->register_file_usage_update(name, update)) {
+    exec_point()->disable_shortcutting_bubble_up("Could not register mktemp()", *name);
+  }
+  return 0;
+}
+
 int Process::handle_faccessat(const int dirfd, const char * const ar_name, const size_t ar_name_len,
                               const int mode, const int flags, const int error) {
   TRACKX(FB_DEBUG_PROC, 1, 1, Process, this,
@@ -763,17 +848,60 @@ int Process::handle_fchmodat(const int fd, const char * const ar_name, const siz
   return 0;
 }
 
+int Process::handle_shm_open(const char * const name, const int oflag,
+                             const mode_t mode, const int fd, const int error) {
+  TRACKX(FB_DEBUG_PROC, 1, 1, Process, this, "name=%s, oflag=%d, mode=%d, fd=%d, error=%d",
+         name, oflag, mode, fd, error);
+  (void)name;
+  (void)oflag;
+  (void)mode;
+
+  if (fd == -1) {
+    switch (error) {
+      case ENAMETOOLONG:
+        /* This does not affect ability to shortcut*/
+        return 0;
+      default:
+        break;
+    }
+#ifdef __APPLE__
+  } else {
+    if (strcmp("com.apple.featureflags.shm", name) == 0
+        || strcmp("apple.shm.notification_center", name) == 0
+        || strncmp("apple.cfprefs", name, strlen("apple.cfprefs")) == 0) {
+      /* Register opened fd and don't disable shortcutting. */
+      // TODO(rbalint) check contents of the shared memory and possibly
+      // include parts in the fingerprint
+      add_filefd(fd, std::make_shared<FileFD>(oflag | O_CLOEXEC, FD_SPECIAL, this));
+      return 0;
+    }
+#endif
+  }
+  exec_point()->disable_shortcutting_bubble_up("shm_open() is not supported");
+  return 0;
+}
+
+#ifdef __APPLE__
+int Process::handle_kqueue(const int fd, const int error) {
+  TRACKX(FB_DEBUG_PROC, 1, 1, Process, this, "fd=%d, error=%d", fd, error);
+  if (!error) {
+    add_filefd(fd, std::make_shared<FileFD>(0, FD_SPECIAL, this))->set_close_on_fork(true);
+  }
+  return 0;
+}
+#endif
+
 #ifdef __linux__
 int Process::handle_memfd_create(const int flags, const int fd) {
   TRACKX(FB_DEBUG_PROC, 1, 1, Process, this, "flags=%d, fd=%d", flags, fd);
-  add_filefd(fd, std::make_shared<FileFD>(fd, (flags & MFD_CLOEXEC) ? O_CLOEXEC : 0,
+  add_filefd(fd, std::make_shared<FileFD>((flags & MFD_CLOEXEC) ? O_CLOEXEC : 0,
                                           FD_SPECIAL, this));
   return 0;
 }
 
 int Process::handle_timerfd_create(const int flags, const int fd) {
   TRACKX(FB_DEBUG_PROC, 1, 1, Process, this, "flags=%d, fd=%d", flags, fd);
-  add_filefd(fd, std::make_shared<FileFD>(fd, (flags & TFD_CLOEXEC) ? O_CLOEXEC : 0,
+  add_filefd(fd, std::make_shared<FileFD>((flags & TFD_CLOEXEC) ? O_CLOEXEC : 0,
                                           FD_SPECIAL, this));
   return 0;
 }
@@ -781,7 +909,7 @@ int Process::handle_timerfd_create(const int flags, const int fd) {
 #ifdef __linux__
 int Process::handle_epoll_create(const int flags, const int fd) {
   TRACKX(FB_DEBUG_PROC, 1, 1, Process, this, "flags=%d, fd=%d", flags, fd);
-  add_filefd(fd, std::make_shared<FileFD>(fd, (flags & EPOLL_CLOEXEC) ? O_CLOEXEC : 0,
+  add_filefd(fd, std::make_shared<FileFD>((flags & EPOLL_CLOEXEC) ? O_CLOEXEC : 0,
                                           FD_SPECIAL, this));
   return 0;
 }
@@ -789,7 +917,7 @@ int Process::handle_epoll_create(const int flags, const int fd) {
 
 int Process::handle_eventfd(const int flags, const int fd) {
   TRACKX(FB_DEBUG_PROC, 1, 1, Process, this, "flags=%d, fd=%d", flags, fd);
-  add_filefd(fd, std::make_shared<FileFD>(fd, (flags & EFD_CLOEXEC) ? O_CLOEXEC : 0,
+  add_filefd(fd, std::make_shared<FileFD>((flags & EFD_CLOEXEC) ? O_CLOEXEC : 0,
                                           FD_SPECIAL, this));
   return 0;
 }
@@ -797,7 +925,7 @@ int Process::handle_eventfd(const int flags, const int fd) {
 int Process::handle_signalfd(const int oldfd, const int flags, const int newfd) {
   TRACKX(FB_DEBUG_PROC, 1, 1, Process, this, "oldfd=%d, flags=%d newfd=%d", oldfd, flags, newfd);
   if (oldfd == -1) {
-    add_filefd(newfd, std::make_shared<FileFD>(newfd, (flags & SFD_CLOEXEC) ? O_CLOEXEC : 0,
+    add_filefd(newfd, std::make_shared<FileFD>((flags & SFD_CLOEXEC) ? O_CLOEXEC : 0,
                                                FD_SPECIAL, this));
   } else {
     /* Reusing old fd, nothing to do.*/
@@ -935,9 +1063,9 @@ void Process::handle_pipe_fds(const int fd0, const int fd1) {
     auto pipe = (new Pipe(pending_pipe->fd0, this))->shared_ptr();
 #endif
     add_filefd(fd0, std::make_shared<FileFD>(
-        fd0, (pending_pipe->flags & ~O_ACCMODE) | O_RDONLY, pipe->fd0_shared_ptr(), this, false));
+        (pending_pipe->flags & ~O_ACCMODE) | O_RDONLY, pipe->fd0_shared_ptr(), this, false));
 
-    auto ffd1 = std::make_shared<FileFD>(fd1, (pending_pipe->flags & ~O_ACCMODE) | O_WRONLY,
+    auto ffd1 = std::make_shared<FileFD>((pending_pipe->flags & ~O_ACCMODE) | O_WRONLY,
                                          pipe->fd1_shared_ptr(),
                                          this, false);
     add_filefd(fd1, ffd1);
@@ -978,7 +1106,7 @@ void Process::handle_socket(const int domain, const int type, const int protocol
         | ((type & SOCK_NONBLOCK) ? O_NONBLOCK : 0)
 #endif
       | 0;
-    add_filefd(ret, std::make_shared<FileFD>(ret, flags, FD_SPECIAL, this));
+    add_filefd(ret, std::make_shared<FileFD>(flags, FD_SPECIAL, this));
 #if defined(SOCK_CLOEXEC) && defined(SOCK_NONBLOCK)
     switch (type & ~(SOCK_CLOEXEC | SOCK_NONBLOCK)) {
 #elif defined(SOCK_CLOEXEC)
@@ -1045,7 +1173,7 @@ void Process::handle_socketpair(const int domain, const int type, const int prot
           | ((type & SOCK_NONBLOCK) ? O_NONBLOCK : 0)
 #endif
         | 0;
-      add_filefd(fd, std::make_shared<FileFD>(fd, flags, FD_SPECIAL, this));
+      add_filefd(fd, std::make_shared<FileFD>(flags, FD_SPECIAL, this));
     }
   } else {
     /* This is ulikely to happen and may not be deterministic .*/
@@ -1053,6 +1181,31 @@ void Process::handle_socketpair(const int domain, const int type, const int prot
   }
 }
 
+void Process::handle_connect(const int sockfd, const char * const address, const size_t address_len,
+                             const int error) {
+  TRACKX(FB_DEBUG_PROC, 1, 1, Process, this, "sockfd=%d, address=%s, error=%d",
+         sockfd, address ? address : "<not set>", error);
+  if (!error) {
+    if (!get_fd(sockfd)) {
+      exec_point()->disable_shortcutting_bubble_up(
+          "Process connect()-ed over and fd that was not tracked as being open, "
+          "which means interception missed at least one socket()", sockfd);
+    } else if (address) {
+      if (is_canonical(address, address_len)
+          && get_absolute(AT_FDCWD, address, address_len)->is_in_ignore_location()) {
+        FB_DEBUG(FB_DEBUG_PROC, std::string("Allow shortcutting skipping logging to ") + address);
+        /* Ignore connect() for logging. Xcodebuild logs execution for example. */
+        return;
+      } else {
+        exec_point()->disable_shortcutting_bubble_up(
+            "connect() to the following address is not supported:", std::string(address));
+        return;
+      }
+    }
+  }
+  exec_point()->disable_shortcutting_bubble_up("connect() is not supported");
+}
+
 int Process::handle_dup3(const int oldfd, const int newfd, const int flags,
                          const int error) {
   TRACKX(FB_DEBUG_PROC, 1, 1, Process, this, "oldfd=%d, newfd=%d, flags=%d, error=%d",
@@ -1088,7 +1241,7 @@ int Process::handle_dup3(const int oldfd, const int newfd, const int flags,
 
   handle_force_close(newfd);
 
-  add_filefd(newfd, std::make_shared<FileFD>(newfd, (*fds_)[oldfd], flags & O_CLOEXEC));
+  add_filefd(newfd, std::make_shared<FileFD>((*fds_)[oldfd], flags & O_CLOEXEC));
   return 0;
 }
 
@@ -1237,6 +1390,25 @@ int Process::handle_fcntl(const int fd, const int cmd, const int arg,
         (*fds_)[fd]->set_cloexec(arg & FD_CLOEXEC);
       }
       return 0;
+#ifdef F_GETPATH
+    case F_GETPATH:
+      if (error == 0) {
+        FileFD *file_fd = get_fd(fd);
+        if (!file_fd) {
+          exec_point()->disable_shortcutting_bubble_up(
+              "Process successfully fcntl'ed on fd which is known to be closed, "
+              "which means interception missed at least one open()", fd);
+          return -1;
+        } else {
+          /* F_GETPATH was successful. It is OK, since the process can be shortcut only
+           * with the process opening the file descriptor already. */
+          return 0;
+        }
+      } else {
+        /* F_GETPATH on an fd failed, this does not affect shortcutting. */
+        return 0;
+      }
+#endif
     default:
       exec_point()->disable_shortcutting_bubble_up("Process executed unsupported fcntl ", d(cmd));
       return 0;
@@ -1292,7 +1464,7 @@ void Process::handle_read_from_inherited(const int fd, const bool is_pread) {
         " missed at least one open()", fd);
     return;
   } else if (file_fd->type() == FD_PIPE_IN
-             && file_fd->pipe() && file_fd->fd() == exec_point()->jobserver_fd_r()) {
+             && file_fd->pipe() && fd == exec_point()->jobserver_fd_r()) {
     // TODO(rbalint) add further check that the process chain did not reopen the fd breaking the
     // connection to the jobserver
     /* It is OK to read from the jobserver. It should not affect the build results. */
@@ -1326,7 +1498,7 @@ void Process::handle_write_to_inherited(const int fd, const bool is_pwrite) {
         " missed at least one open()", fd);
     return;
   } else if (file_fd->type() == FD_PIPE_OUT
-             && file_fd->pipe() && file_fd->fd() == exec_point()->jobserver_fd_w()) {
+             && file_fd->pipe() && fd == exec_point()->jobserver_fd_w()) {
     // TODO(rbalint) add further check that the process chain did not reopen the fd breaking the
     // connection to the jobserver
     /* It is OK to write to the jobserver, but jobserver communication should not be cached. */
@@ -1398,8 +1570,11 @@ void Process::handle_inherited_fd_offset(const int fd, const int64_t offset) {
         "Process reported offset for an intercepted seekable fd which is no known to be open", fd);
     return;
   }
-  exec_point()->disable_shortcutting_only_this(
-      "Inherited writable non-append fd not seeked to its end");
+  const int flags = file_fd->flags() & O_ACCMODE;
+  if (flags == O_WRONLY || flags == O_RDWR) {
+    exec_point()->disable_shortcutting_only_this(
+        "Inherited writable non-append fd not seeked to its end");
+  }
   for (inherited_file_t& inherited_file : exec_point()->inherited_files()) {
     if (inherited_file.fds[0] == fd) {
       inherited_file.start_offset = offset;
@@ -1418,7 +1593,7 @@ void Process::handle_recvmsg_scm_rights(const bool cloexec, const std::vector<in
         "Process successfully received fd via SCM_RIGHTS which is known to be open, which means"
         " interception missed at least one close()", fd);
     } else {
-      add_filefd(fd, std::make_shared<FileFD>(fd, cloexec ? O_CLOEXEC : 0, FD_SCM_RIGHTS, this));
+      add_filefd(fd, std::make_shared<FileFD>(cloexec ? O_CLOEXEC : 0, FD_SCM_RIGHTS, this));
     }
   }
 
@@ -1602,12 +1777,12 @@ Process::pop_expected_child_fds(const std::vector<std::string>& argv,
   TRACKX(FB_DEBUG_PROC, 1, 1, Process, this, "failed=%s", D(failed));
 
   if (expected_child_) {
-    if ((expected_child_->launch_type() == LAUNCH_TYPE_SYSTEM
-         && expected_child_->argv().size() == 0)
+    const LaunchType child_launch_type = expected_child_->launch_type();
+    if ((child_launch_type == LAUNCH_TYPE_SYSTEM && expected_child_->argv().size() == 0)
         || (argv_matches_expectation(argv, expected_child_->argv()))) {
       auto fds = expected_child_->pop_fds();
       if (launch_type_p)
-          *launch_type_p = expected_child_->launch_type();
+          *launch_type_p = child_launch_type;
       if (type_flags_p)
           *type_flags_p = expected_child_->type_flags();
       delete(expected_child_);
