@@ -61,8 +61,7 @@ class ProcessFBBAdaptor {
 
   static int handle(Process *proc, const FBBCOMM_Serialized_dlopen *msg, int fd_conn, int ack_num) {
     return proc->handle_dlopen(
-        msg->has_absolute_filename() ? msg->get_absolute_filename() : nullptr,
-        msg->has_absolute_filename() ? msg->get_absolute_filename_len() : 0,
+        msg->get_libs_as_vector(),
         msg->has_filename() ? msg->get_filename() : nullptr,
         msg->has_filename() ? msg->get_filename_len() : 0,
         msg->get_error(), fd_conn, ack_num);
@@ -81,6 +80,15 @@ class ProcessFBBAdaptor {
     const int error = msg->get_error_no_with_fallback(0);
     return proc->handle_close_range(msg->get_first(), msg->get_last(), msg->get_flags(), error);
   }
+
+  static int handle(Process *proc, const FBBCOMM_Serialized_scandirat *msg) {
+    const int error = msg->get_error_no_with_fallback(0);
+    return proc->handle_scandirat(msg->get_dirfd_with_fallback(AT_FDCWD),
+                                  msg->has_dirp() ? msg->get_dirp() : nullptr,
+                                  msg->has_dirp() ? msg->get_dirp_len() : 0,
+                                  error);
+  }
+
   static int handle(Process *proc, const FBBCOMM_Serialized_truncate *msg) {
     const int error = msg->get_error_no_with_fallback(0);
     return proc->handle_truncate(msg->get_pathname(), msg->get_pathname_len(),
@@ -136,6 +144,20 @@ class ProcessFBBAdaptor {
                                  mode, flags, error);
   }
 
+  static int handle(Process *proc, const FBBCOMM_Serialized_shm_open *msg) {
+    int error = msg->get_error_no_with_fallback(0);
+    int ret = msg->get_ret_with_fallback(-1);
+    return proc->handle_shm_open(msg->get_name(), msg->get_oflag(),
+                                 msg->get_mode_with_fallback(0), ret, error);
+  }
+
+#ifdef __APPLE__
+  static int handle(Process *proc, const FBBCOMM_Serialized_kqueue *msg) {
+    return proc->handle_kqueue(msg->get_ret_with_fallback(-1),
+                               msg->get_error_no_with_fallback(0));
+  }
+#endif
+
 #ifdef __linux__
   static int handle(Process *proc, const FBBCOMM_Serialized_memfd_create *msg) {
     return proc->handle_memfd_create(msg->get_flags(), msg->get_ret());
@@ -274,6 +296,14 @@ class ProcessFBBAdaptor {
     return 0;
   }
 
+  static int handle(Process *proc, const FBBCOMM_Serialized_connect *msg) {
+    proc->handle_connect(msg->get_sockfd(),
+                         msg->get_addr(),
+                         msg->get_addr_len(),
+                         msg->get_error_no_with_fallback(0));
+    return 0;
+  }
+
   static int handle(Process *proc, const FBBCOMM_Serialized_statfs *msg) {
     if (msg->has_pathname()) {
       return proc->handle_statfs(msg->get_pathname(), msg->get_pathname_len(),
@@ -283,6 +313,10 @@ class ProcessFBBAdaptor {
     }
   }
 
+  static int handle(Process *proc, const FBBCOMM_Serialized_mktemp *msg) {
+    return proc->handle_mktemp(msg->get_template(), msg->get_template_len());
+  }
+
  private:
   DISALLOW_COPY_AND_ASSIGN(ProcessFBBAdaptor);
 };
