@@ -31,7 +31,6 @@
 /* Avoid typos in repetitive names */
 #define FB_INSERT_TRACE_MARKERS "FB_INSERT_TRACE_MARKERS"
 #define FB_SOCKET               "FB_SOCKET"
-#define LIBFIREBUILD_SO_LEN     ((int) strlen(LIBFIREBUILD_SO))
 
 /* Like getenv(), but from a custom environment array */
 static char *getenv_from(char **env, const char *name) {
@@ -65,25 +64,15 @@ static bool fb_socket_needs_fixup(char **env) {
   return current_value == NULL || strcmp(current_value, fb_conn_string) != 0;
 }
 
-static bool ld_library_path_needs_fixup(char **env) {
-  if (env_ld_library_path[0] == '\0') {
-    return false;
-  }
-  char *current_value = getenv_from(env, LD_LIBRARY_PATH);
-
-  // FIXME use more precise search (split both values at ':' or ';', and compare the components)
-  return current_value == NULL || strstr(current_value, env_ld_library_path) == NULL;
-}
-
 static bool ld_preload_needs_fixup(char **env) {
   char *current_value = getenv_from(env, LD_PRELOAD);
   if (current_value == NULL) {
     return true;
   }
 
-  const char *loc = strstr(current_value, LIBFIREBUILD_SO);
+  const char *loc = strstr(current_value, libfirebuild_so);
   if (loc) {
-    const char *loc_end = loc + LIBFIREBUILD_SO_LEN;
+    const char *loc_end = loc + libfirebuild_so_len;
     if ((loc == current_value || *(loc - 1) == ':' || *(loc - 1) == ' ')
         && (*loc_end == '\0' || *loc_end == ':' || *loc_end == ' ')) {
       return false;
@@ -98,7 +87,6 @@ bool env_needs_fixup(char **env) {
    * for correctness, just for improving performance a bit. */
   return (fb_insert_trace_markers_needs_fixup(env) ||
           fb_socket_needs_fixup(env) ||
-          ld_library_path_needs_fixup(env) ||
           ld_preload_needs_fixup(env));
 }
 
@@ -121,12 +109,7 @@ int get_env_fixup_size(char **env) {
   ret += strlen(FB_SOCKET "=") + strlen(fb_conn_string) + 1;
 
   char *e = getenv_from(env, LD_PRELOAD);
-  ret += strlen(LD_PRELOAD "=") + (e ? strlen(e) : 0) + 1 + LIBFIREBUILD_SO_LEN + 1;
-
-  e = getenv_from(env, LD_LIBRARY_PATH);
-  ret += strlen(LD_LIBRARY_PATH "=") +
-      (e ? strlen(e) : 0) + 1 +
-      (env_ld_library_path[0] != '\0' ? strlen(env_ld_library_path) : 0) + 1;
+  ret += strlen(LD_PRELOAD "=") + (e ? strlen(e) : 0) + 1 + libfirebuild_so_len + 1;
 
   return ret;
 }
@@ -161,23 +144,6 @@ static int fixup_fb_socket(char *p) {
   return offset + 1;
 }
 
-/* Places the desired value of the LD_LIBRARY_PATH env var
- * (including the "LD_LIBRARY_PATH=" prefix) at @p.
- * The desired value depends on the @current_value.
- *
- * Returns the number of bytes placed (including the trailing NUL).
- */
-static int fixup_ld_library_path(const char *current_value, char *p) {
-  insert_debug_msg("Fixing up LD_LIBRARY_PATH in the environment");
-  int offset;
-  if (current_value == NULL) {
-    sprintf(p, "%s=%s%n", LD_LIBRARY_PATH, env_ld_library_path, &offset);  /* NOLINT */
-  } else {
-    sprintf(p, "%s=%s:%s%n", LD_LIBRARY_PATH, current_value, env_ld_library_path, &offset);  /* NOLINT */
-  }
-  return offset + 1;
-}
-
 /* Places the desired value of the LD_PRELOAD env var
  * (including the "LD_PRELOAD=" prefix) at @p.
  * The desired value depends on the @current_value.
@@ -193,10 +159,10 @@ static int fixup_ld_preload(const char *current_value, char *p) {
   insert_debug_msg("Fixing up LD_PRELOAD in the environment");
   int offset;
   if (current_value == NULL) {
-    sprintf(p, "%s=%s%n", LD_PRELOAD, LIBFIREBUILD_SO, &offset);  /* NOLINT */
+    sprintf(p, "%s=%s%n", LD_PRELOAD, libfirebuild_so, &offset);  /* NOLINT */
   } else {
     /* Append the library. */
-    sprintf(p, "%s=%s:%s%n", LD_PRELOAD, current_value, LIBFIREBUILD_SO, &offset);  /* NOLINT */
+    sprintf(p, "%s=%s:%s%n", LD_PRELOAD, current_value, libfirebuild_so, &offset);  /* NOLINT */
   }
   return offset + 1;
 }
@@ -219,7 +185,6 @@ void env_fixup(char **env, void *buf) {
 
   bool fb_insert_trace_markers_fixed_up = false;
   bool fb_socket_fixed_up = false;
-  bool ld_library_path_fixed_up = false;
   bool ld_preload_fixed_up = false;
 
 #ifdef FB_EXTRA_DEBUG
@@ -243,16 +208,6 @@ void env_fixup(char **env, void *buf) {
     fb_socket_fixed_up = true;
   }
 
-  /* Fix up LD_LIBRARY_PATH if needed */
-  if (ld_library_path_needs_fixup(env)) {
-    char *current_value = getenv_from(env, LD_LIBRARY_PATH);
-    int size = fixup_ld_library_path(current_value, buf2);
-    assert(size > 0);
-    *buf1++ = buf2;
-    buf2 += size;
-    ld_library_path_fixed_up = true;
-  }
-
   /* Fix up LD_PRELOAD if needed */
   if (ld_preload_needs_fixup(env)) {
     char *current_value = getenv_from(env, LD_PRELOAD);
@@ -270,13 +225,13 @@ void env_fixup(char **env, void *buf) {
   for (i = 0; env[i] != NULL; i++) {
     if ((fb_insert_trace_markers_fixed_up && begins_with(env[i], FB_INSERT_TRACE_MARKERS "=")) ||
         (fb_socket_fixed_up && begins_with(env[i], FB_SOCKET "=")) ||
-        (ld_library_path_fixed_up && begins_with(env[i], LD_LIBRARY_PATH "=")) ||
         (ld_preload_fixed_up && begins_with(env[i], LD_PRELOAD "="))) {
       continue;
     }
     *buf1++ = env[i];
   }
 
+  qsort(buf, buf1 - (char**)buf, sizeof(char**), cmpstringpp);
   *buf1 = NULL;
 }
 
@@ -292,12 +247,12 @@ void env_purge(char **env) {
     }
     if (begins_with(env[i], LD_PRELOAD "=")) {
       /* Clear libfirebuild.so */
-      if (strcmp(env[i], LD_PRELOAD "=" LIBFIREBUILD_SO) == 0) {
+      if (strcmp(env[i] + strlen(LD_PRELOAD "="), libfirebuild_so) == 0) {
         /* Just skip LD_PRELOAD. */
         continue;
       } else {
-        char * start = strstr(env[i], LIBFIREBUILD_SO);
-        size_t move_len = LIBFIREBUILD_SO_LEN;
+        char * start = strstr(env[i], libfirebuild_so);
+        size_t move_len = libfirebuild_so_len;
         if (start) {
           if (*(start - 1) == ':' || *(start - 1) == ' ') {
             /* Clear separator before. */
