@@ -74,6 +74,7 @@
 ###     if not syscall
 ###       if target == "darwin"
 #define get_ic_orig_{{ func }}() {{ func }}
+extern {{ rettype }} {{ func }}({{ sig_str }});
 ###       else
 extern {{ rettype }} (*get_ic_orig_{{ func }}(void)) ({{ sig_str }});
 ###       endif
@@ -159,7 +160,7 @@ _{{ func }}
 ###   macro grab_lock_if_needed(grab_condition)
   /* Grabbing the global lock (unless it's already ours, e.g. we're in a signal handler) */
   bool i_locked = false;  /* "i" as in "me, myself and I" */
-  if ({{ grab_condition }}) {
+  if (i_am_intercepting && ({{ grab_condition }})) {
     grab_global_lock(&i_locked, "{{ func }}");
   }
   /* Global lock grabbed */
@@ -191,25 +192,23 @@ _{{ func }}
 ###           endfor
 ###         endif
 
-/* Undefine potential macro */
-#ifdef {{ func }}
-#undef {{ func }}
-#endif
-
 ###       if target == "darwin"
 {{ rettype }} {{ func }} ({{ sig_str }});
 {{ rettype }} interposing_{{ func }} ({{ sig_str }}) {
 ###       else
-{{ rettype }} {{ func }} ({{ sig_str }}) {
+{{ rettype }} interposing_{{ func }} ({{ sig_str }}) __asm__("{{ func }}");
+{{ rettype }} interposing_{{ func }} ({{ sig_str }}) {
 ###       endif
 ###     else
 #ifdef {{ func }}  /* this is prone against typos in the syscall name, but handles older kernels */
 case {{ func }}: {
 #define IC_SYSCALL_{{ func }}_IS_INTERCEPTED
 /* The 64 bit variant has to be defined earlier. */
+###       if not func.endswith("64")
 #if defined {{ func }}64 && !defined IC_SYSCALL_{{ func }}64_IS_INTERCEPTED
 #error "Missing {{ func }}64 interception"
 #endif
+###       endif
   va_list ap_args;
   va_start(ap_args, number);
 ###       for arg in args
@@ -230,8 +229,21 @@ case {{ func }}: {
 ###     if rettype != 'void'
   {{ rettype }} ret;
 ###     endif
-  bool i_am_intercepting = intercepting_enabled;  /* use a copy, in case another thread modifies it */
+
+  /* Maybe don't intercept? */
+###     block intercept
+  /* use a copy, in case another thread modifies it */
+###       if target == "darwin"
+  /* On Darwin the libc calls out to intercepted functions, thus intercept only
+   * the first libc entry point. */
+  bool i_am_intercepting = intercepting_enabled
+                           && (!FB_THREAD_LOCAL(intercept_on)
+                               || FB_THREAD_LOCAL(interception_recursion_depth) > 0);
+###       else
+  bool i_am_intercepting = intercepting_enabled;
+###       endif
   (void)i_am_intercepting;  /* sometimes it's unused, silence warning */
+###     endblock intercept
 
   /* Guard the communication channel */
 ###     block guard_connection_fd
@@ -257,16 +269,12 @@ case {{ func }}: {
 ###       endif
 ###     endif
 
-  /* Maybe don't intercept? */
-###     block no_intercept
-###     endblock no_intercept
-
   /* Warm up */
 ###     if not no_saved_errno
   int saved_errno = errno;
 ###     endif
 
-  if (!ic_init_done) fb_ic_init();
+  if (i_am_intercepting && !ic_init_done) fb_ic_init();
 
 #ifdef FB_EXTRA_DEBUG
   if (insert_trace_markers) {
@@ -317,7 +325,7 @@ case {{ func }}: {
 ###       if not no_saved_errno
   saved_errno = errno;
 ###       endif
-  success = {{ success }};
+  success = ({{ success }});
   (void)success;  /* sometimes it's unused, silence warning */
 
   /* Afterwork */
