@@ -25,13 +25,13 @@
   /* Release the lock, to resemble tpl_exit.c.
    * handle_exit() will re-grab it. */
   thread_signal_danger_zone_enter();
-  if (thread_has_global_lock) {
+  if (FB_THREAD_LOCAL(has_global_lock)) {
     pthread_mutex_unlock(&ic_global_lock);
-    thread_has_global_lock = false;
-    thread_intercept_on = NULL;
+    FB_THREAD_LOCAL(has_global_lock) = false;
+    FB_THREAD_LOCAL(intercept_on) = NULL;
   }
   thread_signal_danger_zone_leave();
-  assert(thread_signal_danger_zone_depth == 0);
+  assert(FB_THREAD_LOCAL(signal_danger_zone_depth) == 0);
 
   /* Mark the end now */
   insert_end_marker("{{ func }}");
