@@ -109,9 +109,8 @@
 
     /* Get CPU time used up to this exec() */
     struct rusage ru;
-    get_ic_orig_getrusage()(RUSAGE_SELF, &ru);
-    timersub(&ru.ru_stime, &initial_rusage.ru_stime, &ru.ru_stime);
-    timersub(&ru.ru_utime, &initial_rusage.ru_utime, &ru.ru_utime);
+    rusage_since_exec(&ru);
+    reset_rusage();
     fbbcomm_builder_exec_set_utime_u(&ic_msg,
         (int64_t)ru.ru_utime.tv_sec * 1000000 + (int64_t)ru.ru_utime.tv_usec);
     fbbcomm_builder_exec_set_stime_u(&ic_msg,
