@@ -29,7 +29,9 @@
      * Request the supervisor to create an intercepted unnamed pipe for us. */
     FBBCOMM_Builder_pipe_request ic_msg1;
     fbbcomm_builder_pipe_request_init(&ic_msg1);
-    fbbcomm_builder_pipe_request_set_flags(&ic_msg1, flags);
+    if (flags != 0) {
+      fbbcomm_builder_pipe_request_set_flags(&ic_msg1, flags);
+    }
     fb_fbbcomm_send_msg(&ic_msg1, fb_sv_conn);
 
     /* Step 2/3. Receive the response from the supervisor, which carries
@@ -79,7 +81,13 @@
 #ifndef NDEBUG
     received =
 #endif
-        TEMP_FAILURE_RETRY(get_ic_orig_recvmsg()(fb_sv_conn, &msgh, (flags & O_CLOEXEC) ? MSG_CMSG_CLOEXEC : 0));
+        TEMP_FAILURE_RETRY(
+#if defined(_TIME_BITS) && (_TIME_BITS == 64)
+            get_ic_orig___recvmsg64()(
+#else
+            get_ic_orig_recvmsg()(
+#endif
+                fb_sv_conn, &msgh, (flags & O_CLOEXEC) ? MSG_CMSG_CLOEXEC : 0));
     assert(received >= 0 && received == (ssize_t)sv_msg_hdr.msg_size);
     assert(fbbcomm_serialized_get_tag((FBBCOMM_Serialized *) sv_msg_buf) == FBBCOMM_TAG_pipe_created);
 
